@@ -1,16 +1,16 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 ### BEGIN LICENSE
 # Copyright (c) 2012, Peter Levi <peterlevi@peterlevi.com>
-# This program is free software: you can redistribute it and/or modify it 
-# under the terms of the GNU General Public License version 3, as published 
+# This program is free software: you can redistribute it and/or modify it
+# under the terms of the GNU General Public License version 3, as published
 # by the Free Software Foundation.
-# 
-# This program is distributed in the hope that it will be useful, but 
-# WITHOUT ANY WARRANTY; without even the implied warranties of 
-# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranties of
+# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 # PURPOSE.  See the GNU General Public License for more details.
-# 
-# You should have received a copy of the GNU General Public License along 
+#
+# You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 ### END LICENSE
 
@@ -29,24 +29,27 @@ class QuoteWriter:
     @staticmethod
     def write_quote(quote, author, infile, outfile, options=None):
         done_event = threading.Event()
-        w, h = Util.get_scaled_size(infile)
         exception = [None]
+
         def go():
             try:
+                w, h = Util.get_scaled_size(infile)
                 surface = QuoteWriter.load_cairo_surface(infile, w, h)
                 QuoteWriter.write_quote_on_surface(surface, quote, author, options)
                 QuoteWriter.save_cairo_surface(surface, outfile)
-            except Exception, e:
+            except Exception as e:
                 exception[0] = e
             finally:
                 done_event.set()
+
         GObject.idle_add(go)
         done_event.wait()
         if exception[0]:
-            raise exception[0]
+            raise exception[0]  # pylint: disable=raising-bad-type
 
     @staticmethod
     def load_cairo_surface(filename, w, h):
+        # pylint: disable=no-member
         pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(filename, w, h, False)
         surface = cairo.ImageSurface(0, pixbuf.get_width(), pixbuf.get_height())
         context = cairo.Context(surface)
@@ -56,14 +59,22 @@ class QuoteWriter:
 
     @staticmethod
     def save_cairo_surface(surface, filename):
-        size = surface.get_width(), surface.get_height()
-        image = Image.frombuffer('RGBA', size, surface.get_data(), 'raw', 'BGRA', 0, 1)
-        image.save(filename, quality=100)
+        try:
+            # attempt faster method first
+            # the get_data() call will fail with Cairo version < 1.15.4-1 (e.g. on 16.04)
+            # https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=809479
+            data = surface.get_data()
+            size = surface.get_width(), surface.get_height()
+            image = Image.frombuffer('RGBA', size, data.tobytes(), 'raw', 'BGRA', 0, 1).convert("RGB")
+            image.save(filename, quality=100)
+        except:
+            # fallback to slower method, but which works on 16.04
+            surface.write_to_png(filename)
 
     @staticmethod
     def write_quote_on_surface(surface, quote, author=None, options=None, margin=30):
-        qcontext = cairo.Context(surface)
-        acontext = cairo.Context(surface)
+        qcontext = cairo.Context(surface)  # pylint: disable=no-member
+        acontext = cairo.Context(surface)  # pylint: disable=no-member
 
         iw = surface.get_width()
         ih = surface.get_height()
@@ -144,4 +155,4 @@ if __name__ == "__main__":
         '"I may be drunk, Miss, but in the morning I will be sober and you will still be ugly."',
         "Winston Churchill",
         "test.jpg",
-        "test_result.jpg")
+        "test_result.png")
